import { __decorate, __spread } from 'tslib';
import { NgModule, NgZone, ɵɵdefineInjectable, ɵɵinject, Injectable, EventEmitter, Input, Output, ViewChild, Component } from '@angular/core';
import * as YtPlayer from 'yt-player';
import { Subject } from 'rxjs';

var YtPlayerAdapterModule = /** @class */ (function () {
    function YtPlayerAdapterModule() {
    }
    YtPlayerAdapterModule = __decorate([
        NgModule({})
    ], YtPlayerAdapterModule);
    return YtPlayerAdapterModule;
}());

var StateChangeType;
(function (StateChangeType) {
    StateChangeType[StateChangeType["Error"] = 0] = "Error";
    StateChangeType[StateChangeType["Unplayable"] = 1] = "Unplayable";
    StateChangeType[StateChangeType["Unstarted"] = 2] = "Unstarted";
    StateChangeType[StateChangeType["Ended"] = 3] = "Ended";
    StateChangeType[StateChangeType["Started"] = 4] = "Started";
    StateChangeType[StateChangeType["PlaybackProgress"] = 5] = "PlaybackProgress";
    StateChangeType[StateChangeType["Paused"] = 6] = "Paused";
    StateChangeType[StateChangeType["Buffering"] = 7] = "Buffering";
    StateChangeType[StateChangeType["Cued"] = 8] = "Cued";
    StateChangeType[StateChangeType["QualityChanged"] = 9] = "QualityChanged";
    StateChangeType[StateChangeType["RateChanged"] = 10] = "RateChanged";
})(StateChangeType || (StateChangeType = {}));

var EventsRegistry = /** @class */ (function () {
    function EventsRegistry(ngZone) {
        var _this = this;
        this.ngZone = ngZone;
        this.stateChange$ = new Subject();
        this.eventHandlers = new Map([
            ["error", function (err) { return _this.broadcastStatusChange(StateChangeType.Error, err); }],
            [
                "unplayable",
                function (videoId) { return _this.broadcastStatusChange(StateChangeType.Unplayable, videoId); }
            ],
            ["unstarted", function () { return _this.broadcastStatusChange(StateChangeType.Unstarted); }],
            ["ended", function () { return _this.broadcastStatusChange(StateChangeType.Ended); }],
            ["playing", function () { return _this.broadcastStatusChange(StateChangeType.Started); }],
            ["paused", function () { return _this.broadcastStatusChange(StateChangeType.Paused); }],
            ["buffering", function () { return _this.broadcastStatusChange(StateChangeType.Buffering); }],
            ["cued", function () { return _this.broadcastStatusChange(StateChangeType.Cued); }],
            [
                "playbackQualityChange",
                function (quality) {
                    return _this.broadcastStatusChange(StateChangeType.QualityChanged, quality);
                }
            ],
            [
                "playbackRateChange",
                function (rate) { return _this.broadcastStatusChange(StateChangeType.RateChanged, rate); }
            ],
            [
                "timeupdate",
                function (time) { return _this.broadcastStatusChange(StateChangeType.PlaybackProgress, time); }
            ]
        ]);
    }
    EventsRegistry.prototype.register = function (player) {
        this.eventHandlers.forEach(function (eventHandler, eventName) {
            player.on(eventName, eventHandler);
        });
    };
    EventsRegistry.prototype.broadcastStatusChange = function (type, payload) {
        var _this = this;
        this.ngZone.run(function () { return _this.stateChange$.next({ type: type, payload: payload }); });
    };
    EventsRegistry.ctorParameters = function () { return [
        { type: NgZone }
    ]; };
    EventsRegistry.ɵprov = ɵɵdefineInjectable({ factory: function EventsRegistry_Factory() { return new EventsRegistry(ɵɵinject(NgZone)); }, token: EventsRegistry, providedIn: YtPlayerAdapterModule });
    EventsRegistry = __decorate([
        Injectable({ providedIn: YtPlayerAdapterModule })
    ], EventsRegistry);
    return EventsRegistry;
}());

var YtPlayerService = /** @class */ (function () {
    function YtPlayerService(eventsRegistry) {
        this.eventsRegistry = eventsRegistry;
    }
    Object.defineProperty(YtPlayerService.prototype, "stateChange$", {
        get: function () { return this.eventsRegistry.stateChange$.asObservable(); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(YtPlayerService.prototype, "destroyed", {
        get: function () { return this.player.destroyed; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(YtPlayerService.prototype, "videoId", {
        get: function () { return this.player.videoId; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(YtPlayerService.prototype, "player", {
        get: function () { return this.ytPlayer; },
        set: function (value) { this.ytPlayer = value; },
        enumerable: true,
        configurable: true
    });
    YtPlayerService.prototype.init = function (htmlId, playerOptions) {
        this.setUpPlayer(htmlId, playerOptions);
        this.eventsRegistry.register(this.player);
    };
    YtPlayerService.prototype.load = function (videoId) {
        var autoplay = this.playerOptions && this.playerOptions.autoplay;
        this.player.load(videoId, autoplay);
    };
    YtPlayerService.prototype.play = function () {
        this.player.play();
    };
    YtPlayerService.prototype.pause = function () {
        this.player.pause();
    };
    YtPlayerService.prototype.stop = function () {
        this.player.stop();
    };
    YtPlayerService.prototype.seek = function (seconds) {
        this.player.seek(seconds);
    };
    YtPlayerService.prototype.setVolume = function (value) {
        if (value > 100) {
            console.warn("Provided value " + value + " exceeds max value. 100 used instead");
        }
        this.player.setVolume(value);
    };
    YtPlayerService.prototype.getVolume = function () {
        return this.player.getVolume();
    };
    YtPlayerService.prototype.mute = function () {
        this.player.mute();
    };
    YtPlayerService.prototype.unMute = function () {
        this.player.unMute();
    };
    YtPlayerService.prototype.isMuted = function () {
        return this.player.isMuted();
    };
    YtPlayerService.prototype.setSize = function (width, height) {
        this.player.setSize(width, height);
    };
    YtPlayerService.prototype.setPlaybackRate = function (rate) {
        this.player.setPlaybackRate(rate);
    };
    YtPlayerService.prototype.setPlaybackQuality = function (sugestedQuality) {
        this.player.setPlaybackQuality(sugestedQuality);
    };
    YtPlayerService.prototype.getPlaybackRate = function () {
        return this.player.getPlaybackRate();
    };
    YtPlayerService.prototype.getAvailablePlaybackRates = function () {
        return this.player.getAvailablePlaybackRates();
    };
    YtPlayerService.prototype.getDuration = function () {
        return this.player.getDuration();
    };
    YtPlayerService.prototype.getBufferingProgress = function () {
        return this.player.getProgress();
    };
    YtPlayerService.prototype.getState = function () {
        return this.player.getState();
    };
    YtPlayerService.prototype.getCurrentTime = function () {
        return this.player.getCurrentTime();
    };
    YtPlayerService.prototype.destroy = function () {
        this.player.destroy();
    };
    YtPlayerService.prototype.setUpPlayer = function (htmlId, playerOptions) {
        this.playerOptions = playerOptions;
        this.player = new YtPlayer(htmlId, playerOptions);
    };
    YtPlayerService.ctorParameters = function () { return [
        { type: EventsRegistry }
    ]; };
    YtPlayerService.ɵprov = ɵɵdefineInjectable({ factory: function YtPlayerService_Factory() { return new YtPlayerService(ɵɵinject(EventsRegistry)); }, token: YtPlayerService, providedIn: YtPlayerAdapterModule });
    YtPlayerService = __decorate([
        Injectable({ providedIn: YtPlayerAdapterModule })
    ], YtPlayerService);
    return YtPlayerService;
}());

var IdGeneratorService = /** @class */ (function () {
    function IdGeneratorService() {
        this.charCodes = { a: 65, z: 90 };
        this.idLength = 8;
    }
    IdGeneratorService.prototype.generate = function () {
        var _this = this;
        return __spread(Array(this.idLength).keys()).reduce(function (acc) { return "" + acc + _this.getRandomLetter(); }, '');
    };
    IdGeneratorService.prototype.getRandomLetter = function () {
        var charCode = this.getRandomIntInRange(this.charCodes.a, this.charCodes.z);
        return String.fromCharCode(charCode);
    };
    IdGeneratorService.prototype.getRandomIntInRange = function (min, max) {
        return Math.floor(Math.random() * (max - min + 1)) + min;
    };
    IdGeneratorService.ɵprov = ɵɵdefineInjectable({ factory: function IdGeneratorService_Factory() { return new IdGeneratorService(); }, token: IdGeneratorService, providedIn: "root" });
    IdGeneratorService = __decorate([
        Injectable({ providedIn: 'root' })
    ], IdGeneratorService);
    return IdGeneratorService;
}());

var QueueService = /** @class */ (function () {
    function QueueService() {
        this.queue = new Map();
    }
    QueueService.prototype.enqueue = function (task) {
        var key = this.generateKey();
        this.addTask(key, task);
    };
    QueueService.prototype.dequeue = function () {
        var task = this.getAndRemoveFromQueue();
        this.execute(task);
    };
    QueueService.prototype.addTask = function (key, task) {
        this.queue.set(key, task);
    };
    QueueService.prototype.generateKey = function () {
        return this.queue.size + 1;
    };
    QueueService.prototype.getAndRemoveFromQueue = function () {
        var latestKey = this.queue.size;
        var task = this.queue.get(latestKey);
        this.queue.delete(latestKey);
        return task;
    };
    QueueService.prototype.execute = function (task) {
        if (task && typeof task === 'function') {
            task();
        }
    };
    QueueService.ɵprov = ɵɵdefineInjectable({ factory: function QueueService_Factory() { return new QueueService(); }, token: QueueService, providedIn: "root" });
    QueueService = __decorate([
        Injectable({ providedIn: 'root' })
    ], QueueService);
    return QueueService;
}());

var YtPlayerComponent = /** @class */ (function () {
    function YtPlayerComponent(ytPlayerService, idGeneratorService, queueService) {
        this.ytPlayerService = ytPlayerService;
        this.idGeneratorService = idGeneratorService;
        this.queueService = queueService;
        this.stateChange = new EventEmitter();
        this.errorMessage = 'Player not initialized. Did you specify [videoId] input property of yt-player-component?';
    }
    YtPlayerComponent.prototype.ngOnInit = function () {
        this.setUpElementRefId();
        this.dequeuePlayerInitialization();
        this.loadVideo();
        this.subscribeToStateChanges();
    };
    YtPlayerComponent.prototype.ngOnChanges = function () {
        if (!this.videoIdIsDefined()) {
            return;
        }
        else if (this.elementRefIdIsDefined()) {
            this.reloadVideo();
        }
        else {
            this.enqueuePlayerInitialization();
        }
    };
    YtPlayerComponent.prototype.ngOnDestroy = function () {
        this.ytPlayerService.destroy();
        this.stateChangeSubscription.unsubscribe();
    };
    YtPlayerComponent.prototype.setUpElementRefId = function () {
        var id = this.idGeneratorService.generate();
        this.ytHtmlElementHook.nativeElement.setAttribute('id', id);
    };
    YtPlayerComponent.prototype.videoIdIsDefined = function () {
        return !!this.videoId;
    };
    YtPlayerComponent.prototype.elementRefIdIsDefined = function () {
        return (this.ytHtmlElementHook.nativeElement &&
            this.ytHtmlElementHook.nativeElement.getAttribute('id'));
    };
    YtPlayerComponent.prototype.initializePlayer = function () {
        var id = this.ytHtmlElementHook.nativeElement.getAttribute('id');
        this.ytPlayerService.init("#" + id, this.options);
    };
    YtPlayerComponent.prototype.loadVideo = function () {
        try {
            this.ytPlayerService.load(this.videoId);
        }
        catch (err) {
            console.error(this.errorMessage);
        }
    };
    YtPlayerComponent.prototype.reloadVideo = function () {
        this.ytPlayerService.destroy();
        this.initializePlayer();
        this.loadVideo();
    };
    YtPlayerComponent.prototype.enqueuePlayerInitialization = function () {
        var _this = this;
        this.queueService.enqueue(function () { return _this.initializePlayer(); });
    };
    YtPlayerComponent.prototype.dequeuePlayerInitialization = function () {
        this.queueService.dequeue();
    };
    YtPlayerComponent.prototype.subscribeToStateChanges = function () {
        var _this = this;
        this.stateChangeSubscription = this.ytPlayerService.stateChange$.subscribe(function (stateChange) { return _this.stateChange.emit(stateChange); });
    };
    YtPlayerComponent.ctorParameters = function () { return [
        { type: YtPlayerService },
        { type: IdGeneratorService },
        { type: QueueService }
    ]; };
    __decorate([
        Input()
    ], YtPlayerComponent.prototype, "videoId", void 0);
    __decorate([
        Input()
    ], YtPlayerComponent.prototype, "options", void 0);
    __decorate([
        Output()
    ], YtPlayerComponent.prototype, "stateChange", void 0);
    __decorate([
        ViewChild('ytHtmlElementHook', { static: true })
    ], YtPlayerComponent.prototype, "ytHtmlElementHook", void 0);
    YtPlayerComponent = __decorate([
        Component({
            selector: 'yt-player',
            template: "\n    <div #ytHtmlElementHook></div>\n  "
        })
    ], YtPlayerComponent);
    return YtPlayerComponent;
}());

var YtPlayerAngularModule = /** @class */ (function () {
    function YtPlayerAngularModule() {
    }
    YtPlayerAngularModule = __decorate([
        NgModule({
            imports: [YtPlayerAdapterModule],
            declarations: [YtPlayerComponent],
            exports: [YtPlayerComponent]
        })
    ], YtPlayerAngularModule);
    return YtPlayerAngularModule;
}());

var PlayerState;
(function (PlayerState) {
    PlayerState[PlayerState["Unstarted"] = 0] = "Unstarted";
    PlayerState[PlayerState["Ended"] = 1] = "Ended";
    PlayerState[PlayerState["Playing"] = 2] = "Playing";
    PlayerState[PlayerState["Paused"] = 3] = "Paused";
    PlayerState[PlayerState["Buffering"] = 4] = "Buffering";
    PlayerState[PlayerState["Cued"] = 5] = "Cued";
})(PlayerState || (PlayerState = {}));

var PlaybackQuality;
(function (PlaybackQuality) {
    PlaybackQuality["Small"] = "small";
    PlaybackQuality["Medium"] = "medium";
    PlaybackQuality["Large"] = "large";
    PlaybackQuality["Hd720"] = "hd720";
    PlaybackQuality["Hd1080"] = "hd1080";
    PlaybackQuality["HighRes"] = "highres";
    PlaybackQuality["Default"] = "default";
})(PlaybackQuality || (PlaybackQuality = {}));

/*
 * Public API Surface of yt-player-angular
 */

/**
 * Generated bundle index. Do not edit.
 */

export { PlaybackQuality, PlayerState, StateChangeType, YtPlayerAngularModule, YtPlayerComponent, YtPlayerService, YtPlayerAdapterModule as ɵa, EventsRegistry as ɵb, IdGeneratorService as ɵc, QueueService as ɵd };
//# sourceMappingURL=yt-player-angular.js.map
